#############################################
# import_pages.py - Apply wxWidgets theme to a web page or pages
# Author: Kevin Ollivier
# License: wxWidgets License
#############################################

import string, sys, os, glob
import PyMeld, copy, re, StringIO
from section_links import *

root_dir = os.path.join("..", "site-v2")
print "Loading templates..."
# First, read in the site template(s)
main_template = open(os.path.join(root_dir, "Templates", "template.dwt")).read()
template_meld = PyMeld.Meld(main_template)

section_file = os.path.join(root_dir, "Templates", "section_template.dwt")
section_template = open(section_file).read()
section_meld = PyMeld.Meld(section_template)

# Update the section template with any changes to the main template
sec_template = copy.deepcopy(template_meld)
sec_template.contents._content = section_meld.contents._content
section_meld = sec_template   

def GetBody(myhtml):
	"""
	We use this function instead of an HTML parser because HTML parsers typically choke on
	non-standard HTML, which is quite common.
	"""
	inbody = 0
	inscript = 0
	bodystart = 0
	bodyend = 0
	text = ""
	uppercase = 1
	html = myhtml.readline()
	while not html == "":
		#if not encoding and string.find(html.lower(), "<meta"):
		#	encoding = GetEncoding(html)
		#if we're inside a script, mark it so that we can test if body tag is inside the script
		scriptstart = string.find(html, "<SCRIPT")
		if scriptstart == -1:
			scriptstart = string.find(html, "<script")

		if not string.find(html, "</SCRIPT>") == -1 or not string.find(html, "</script>") == -1:
			inscript = 0

		#if we've found a script BEFORE the start of the body tag, and then found a body tag
		#it would be part of the script
		#that's why we check the script status first
		if not scriptstart == -1 and inbody == 0:
			inscript = 1

		#check for start of body in upper and lowercase
		bodystart = string.find(string.lower(html), "<body")

		#if body is found, mark the end of it
		if not bodystart == -1:
			bodystart = string.find(html, ">", bodystart)

		#if we've found both a body tag and a script tag, find which one comes first
		#if script is first, this isn't the "real" body tag
		if bodystart != -1 and scriptstart != -1:
			if bodystart > scriptstart:
				inscript = 1

		#if we are not in a script, and we've found the body tag, capture the text
		if inscript == 0 and (not bodystart == -1 or inbody):
			inbody = 1
			bodyend = string.find(string.lower(html), "</body>")
				
			#if both <BODY> and </BODY> are on same line, grab it all
			if not bodystart == -1 and not bodyend == -1:
				text = text + html[bodystart+1:bodyend]
				bodystart = -1
				bodyend = -1
				inbody = 0
			elif not bodyend == -1:
				#if bodyend == 0:
				#	bodyend = 1 #a hack because -1 means everything
				inbody = 0
				text = text + html[0:bodyend] 
				bodyend = -1
			elif not bodystart == -1:
				text = text + html[bodystart+1:-1] 
				bodystart = -1
			elif inbody == 1:
				text = text + html 
		html = myhtml.readline()
	return text

def updateFile(filename):
    """
    This imports the existing page contents (minus the template areas) and
    then applies the latest template to those contents.
    """
    try:
        # Read in the page data we want to keep
        html = open(filename).read()
        title_regex = re.compile("<title>([^<]*)</title>", re.IGNORECASE)
        title = title_regex.search(html).group(0)
        
        contents = GetBody(StringIO.StringIO(html))
        
        print "Updating %s" % (filename)
        
        # Check if it's a sub-section of the site, and if so, add the links
        relative_path = filename.replace(root_dir + "/", "")
        if os.path.isabs(relative_path):
            relative_path = string.replace(os.path.dirname(filename), root_dir, "")[1:]
        
        # Make a copy of the main template, then insert the new values
        global template_meld
        page_template = copy.deepcopy(template_meld)
        
        if relative_path != "":
            section = relative_path.split(os.sep)[0]
            mylinks = links[section]
            global section_meld
            page_template = copy.deepcopy(section_meld)
            
            page_template.section_contents = contents
            #except:
            #    page_template.section_contents = contents
                
            page_template.section_name = section.capitalize()
            page_template.section_links = makeLinksHTML(mylinks)
        
        page_template = title_regex.sub(title, str(page_template))
        if relative_path != "":
            subdirs = len(relative_path.split(os.sep))
            reldir = ".."
            for num in range(1, subdirs-1):
                reldir = reldir + "/.."
            page_template = page_template.replace('href="images', 'href="%s/images' % (reldir))
            page_template = page_template.replace('src="images', 'src="%s/images' % (reldir))
            
        output = open(filename, "w")
        output.write(str(page_template))
        output.close()
    except:
        # just move onto the next file
        sys.stderr.write("WARNING: Unable to update %s" % (filename))
        import traceback
        sys.stderr.write(`traceback.print_exc()`)



# Grab list of files from the command line and run updateFile on them.	
file_list = sys.argv[1:]

for group in file_list:
    files = glob.glob(group)
    for item in files:
        updateFile(item)
    
